@@ -2,7 +2,7 @@ import tempfile
 
 import click
 
-from pymobiledevice3.cli.cli_common import Command, print_json
+from pymobiledevice3.cli.cli_common import Command, print_json, CommandWithoutAutopair
 from pymobiledevice3.lockdown import LockdownClient
 from pymobiledevice3.services.heartbeat import HeartbeatService
 from pymobiledevice3.tcp_forwarder import TcpForwarder
@@ -62,13 +62,41 @@ def lockdown_info(lockdown: LockdownClient, all, color):
     print_json(lockdown.all_domains if all else lockdown.all_values, colored=color)
 
 
-@lockdown_group.command('unpair', cls=Command)
+@lockdown_group.command('get', cls=Command)
+@click.argument('domain', required=False)
+@click.argument('key', required=False)
+@click.option('--color/--no-color', default=True)
+def lockdown_get(lockdown: LockdownClient, domain, key, color):
+    """ query lockdown values by their domain and key names """
+    print_json(lockdown.get_value(domain=domain, key=key), colored=color)
+
+
+@lockdown_group.command('set', cls=Command)
+@click.argument('value')
+@click.argument('domain', required=False)
+@click.argument('key', required=False)
+@click.option('--color/--no-color', default=True)
+def lockdown_set(lockdown: LockdownClient, value, domain, key, color):
+    """ set a lockdown value using python's eval() """
+    print_json(lockdown.set_value(value=eval(value), domain=domain, key=key), colored=color)
+
+
+@lockdown_group.command('remove', cls=Command)
+@click.argument('domain')
+@click.argument('key')
+@click.option('--color/--no-color', default=True)
+def lockdown_remove(lockdown: LockdownClient, domain, key, color):
+    """ remove a domain/key pair """
+    print_json(lockdown.remove_value(domain=domain, key=key), colored=color)
+
+
+@lockdown_group.command('unpair', cls=CommandWithoutAutopair)
 def lockdown_unpair(lockdown: LockdownClient):
     """ unpair from connected device """
     lockdown.unpair()
 
 
-@lockdown_group.command('pair', cls=Command)
+@lockdown_group.command('pair', cls=CommandWithoutAutopair)
 def lockdown_pair(lockdown: LockdownClient):
     """ pair device """
     lockdown.pair()
