@@ -5,7 +5,9 @@ import os
 import platform
 import plistlib
 import sys
+import time
 import uuid
+from contextlib import suppress
 from pathlib import Path
 from typing import Mapping
 
@@ -14,13 +16,15 @@ from packaging.version import Version
 from pymobiledevice3 import usbmux
 from pymobiledevice3.ca import ca_do_everything
 from pymobiledevice3.exceptions import *
+from pymobiledevice3.exceptions import LockdownError, SetProhibitedError
 from pymobiledevice3.service_connection import ServiceConnection
 from pymobiledevice3.usbmux import PlistMuxConnection
 from pymobiledevice3.utils import sanitize_ios_version
 
+SYSTEM_BUID = '30142955-444094379208051516'
+
 # we store pairing records and ssl keys in ~/.pymobiledevice3
 HOMEFOLDER = Path.home() / '.pymobiledevice3'
-MAXTRIES = 20
 
 LOCKDOWN_PATH = {
     'win32': Path(os.environ.get('ALLUSERSPROFILE', ''), 'Apple', 'Lockdown'),
@@ -45,7 +49,7 @@ def reconnect_on_remote_close(f):
     def _reconnect_on_remote_close(*args, **kwargs):
         try:
             return f(*args, **kwargs)
-        except BrokenPipeError:
+        except (BrokenPipeError, ConnectionTerminatedError):
             self = args[0]
 
             # first we release the socket on our end to avoid a ResourceWarning
@@ -100,7 +104,18 @@ class LockdownClient(object):
     DEFAULT_CLIENT_NAME = 'pymobiledevice3'
     SERVICE_PORT = 62078
 
-    def __init__(self, udid=None, client_name=DEFAULT_CLIENT_NAME, autopair=True, connection_type=None):
+    def __init__(self, udid: str = None, client_name: str = DEFAULT_CLIENT_NAME, autopair: bool = True,
+                 connection_type: str = None, pair_timeout: int = None, hostname: str = None,
+                 pair_record: Mapping = None):
+        """
+        :param udid: serial number for device to connect to
+        :param client_name: user agent to use when identifying for lockdownd
+        :param autopair: should automatically attempt pairing with device
+        :param connection_type: can be either "USB" or "Network" to specify what connection type to use
+        :param pair_timeout: if autopair, use this timeout for user's Trust dialog. If None, will wait forever
+        :param hostname: use given hostname to generate the HostID inside the pair record
+        :param pair_record: use this pair record instead of the one already stored
+        """
         device = usbmux.select_device(udid, connection_type=connection_type)
         if device is None:
             if udid:
@@ -111,12 +126,13 @@ class LockdownClient(object):
         self.usbmux_device = device
         self.logger = logging.getLogger(__name__)
         self.paired = False
-        self.SessionID = None
+        self.session_id = None
         self.service = ServiceConnection.create(udid, self.SERVICE_PORT, connection_type=device.connection_type)
-        self.host_id = self.generate_host_id()
-        self.system_buid = None
+        self.host_id = self.generate_host_id(hostname)
+        self.system_buid = SYSTEM_BUID
         self.label = client_name
-        self.pair_record = None
+        self.pair_record = pair_record
+        self.ssl_file = None
 
         if self.query_type() != 'com.apple.mobile.lockdown':
             raise IncorrectModeError()
@@ -125,12 +141,13 @@ class LockdownClient(object):
         self.udid = self.all_values.get('UniqueDeviceID', self.usbmux_device.serial)
         self.unique_chip_id = self.all_values.get('UniqueChipID')
         self.device_public_key = self.all_values.get('DevicePublicKey')
-        self.ios_version = self.all_values.get('ProductVersion')
+        self.product_version = self.all_values.get('ProductVersion')
+        self.product_type = self.all_values.get('ProductType')
         self.identifier = self.udid
 
         if not self.identifier:
             if self.unique_chip_id:
-                self.identifier = '%x' % self.unique_chip_id
+                self.identifier = f'{self.unique_chip_id:x}'
             else:
                 raise PyMobileDevice3Exception('Could not get UDID or ECID, failing')
 
@@ -141,30 +158,34 @@ class LockdownClient(object):
                 # but pairing by default was not requested
                 return
 
-            self.pair()
+            self.pair(timeout=pair_timeout)
+
+            # get session_id
             if not self.validate_pairing():
                 raise FatalPairingError()
-            self.service = ServiceConnection.create(udid, self.SERVICE_PORT,
-                                                    connection_type=self.usbmux_device.connection_type)
 
         # reload data after pairing
         self.all_values = self.get_value()
 
+    def __repr__(self):
+        return f'<{self.__class__.__name__} ID:{self.identifier} VERSION:{self.product_version} ' \
+               f'TYPE:{self.product_type} PAIRED:{self.paired}>'
+
     def __enter__(self):
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         self.close()
 
-    def query_type(self):
-        return self.service.send_recv_plist({'Label': self.label, 'Request': 'QueryType'}).get('Type')
+    def query_type(self) -> str:
+        return self._request('QueryType').get('Type')
 
     @property
     def all_domains(self) -> Mapping:
         result = self.all_values
 
         for domain in DOMAINS:
-            result.update(self.get_value(domain))
+            result.update({domain: self.get_value(domain)})
 
         return result
 
@@ -179,6 +200,26 @@ class LockdownClient(object):
             result[key] = self.all_values.get(key)
         return result
 
+    @property
+    def share_iphone_analytics_enabled(self) -> bool:
+        return self.get_value('com.apple.MobileDeviceCrashCopy', 'ShouldSubmit')
+
+    @property
+    def voice_over(self) -> bool:
+        return bool(self.get_value('com.apple.Accessibility').get('VoiceOverTouchEnabledByiTunes', 0))
+
+    @voice_over.setter
+    def voice_over(self, value: bool):
+        self.set_value(int(value), 'com.apple.Accessibility', 'VoiceOverTouchEnabledByiTunes')
+
+    @property
+    def invert_display(self) -> bool:
+        return bool(self.get_value('com.apple.Accessibility').get('InvertDisplayEnabledByiTunes', 0))
+
+    @invert_display.setter
+    def invert_display(self, value: bool):
+        self.set_value(int(value), 'com.apple.Accessibility', 'InvertDisplayEnabledByiTunes')
+
     @property
     def enable_wifi_connections(self):
         return self.get_value('com.apple.mobile.wireless_lockdown').get('EnableWifiConnections', False)
@@ -209,7 +250,7 @@ class LockdownClient(object):
 
     @property
     def sanitized_ios_version(self):
-        return sanitize_ios_version(self.ios_version)
+        return sanitize_ios_version(self.product_version)
 
     def set_language(self, language: str):
         self.set_value(language, key='Language', domain='com.apple.international')
@@ -218,24 +259,22 @@ class LockdownClient(object):
         self.set_value(locale, key='Locale', domain='com.apple.international')
 
     @staticmethod
-    def generate_host_id() -> str:
-        hostname = platform.node()
+    def generate_host_id(hostname: str = None) -> str:
+        hostname = platform.node() if hostname is None else hostname
         host_id = uuid.uuid3(uuid.NAMESPACE_DNS, hostname)
         return str(host_id).upper()
 
     @reconnect_on_remote_close
     def enter_recovery(self):
-        self.service.send_plist({'Request': 'EnterRecovery'})
-        return self.service.recv_plist()
+        return self._request('EnterRecovery')
 
     def stop_session(self):
-        if self.SessionID and self.service:
-            self.service.send_plist({'Label': self.label, 'Request': 'StopSession', 'SessionID': self.SessionID})
-            self.SessionID = None
-            res = self.service.recv_plist()
-            if not res or res.get('Result') != 'Success':
+        if self.session_id and self.service:
+            response = self._request('StopSession', {'SessionID': self.session_id})
+            self.session_id = None
+            if not response or response.get('Result') != 'Success':
                 raise CannotStopSessionError()
-            return res
+            return response
 
     def get_itunes_pairing_record(self):
         platform_type = 'linux' if not sys.platform.startswith('linux') else sys.platform
@@ -255,40 +294,34 @@ class LockdownClient(object):
             return None
         return plistlib.loads(path.read_bytes())
 
-    def validate_pairing(self):
-        pair_record = self.get_itunes_pairing_record()
-        if pair_record is not None:
-            self.logger.info(f'Using iTunes pair record: {self.identifier}.plist')
-
-        mux = usbmux.create_mux()
-        if isinstance(mux, PlistMuxConnection):
-            pair_record = mux.get_pair_record(self.udid)
-        else:
-            pair_record = self.get_local_pairing_record()
-        mux.close()
-
-        if pair_record is None:
+    def validate_pairing(self) -> bool:
+        try:
+            self._init_preferred_pair_record()
+        except NotPairedError:
             return False
 
-        self.pair_record = pair_record
+        if self.pair_record is None:
+            return False
 
-        cert_pem = pair_record['HostCertificate']
-        private_key_pem = pair_record['HostPrivateKey']
+        cert_pem = self.pair_record['HostCertificate']
+        private_key_pem = self.pair_record['HostPrivateKey']
 
-        if Version(self.ios_version) < Version('11.0'):
-            validate_pair = {'Label': self.label, 'Request': 'ValidatePair', 'PairRecord': pair_record}
-            self.service.send_plist(validate_pair)
-            r = self.service.recv_plist()
-            if (not r) or ('Error' in r):
-                self.logger.error('ValidatePair fail', validate_pair)
+        if Version(self.product_version) < Version('11.0'):
+            try:
+                self._request('ValidatePair', {'PairRecord': self.pair_record})
+            except PairingError:
                 return False
 
-        self.host_id = pair_record.get('HostID', self.host_id)
-        self.system_buid = pair_record.get('SystemBUID', self.system_buid)
-        start_session = self.service.send_recv_plist(
-            {'Label': self.label, 'Request': 'StartSession', 'HostID': self.host_id, 'SystemBUID': self.system_buid}
-        )
-        self.SessionID = start_session.get('SessionID')
+        self.host_id = self.pair_record.get('HostID', self.host_id)
+        self.system_buid = self.pair_record.get('SystemBUID', self.system_buid)
+
+        try:
+            start_session = self._request('StartSession', {'HostID': self.host_id, 'SystemBUID': self.system_buid})
+        except InvalidHostIDError:
+            # no host id means there is no such pairing record
+            return False
+
+        self.session_id = start_session.get('SessionID')
         if start_session.get('EnableSessionSSL'):
             lf = b'\n'
             self.ssl_file = write_home_file(f'{self.identifier}_ssl.txt', cert_pem + lf + private_key_pem)
@@ -298,7 +331,7 @@ class LockdownClient(object):
         return True
 
     @reconnect_on_remote_close
-    def pair(self):
+    def pair(self, timeout: int = None):
         self.device_public_key = self.get_value('', 'DevicePublicKey')
         if not self.device_public_key:
             self.logger.error('Unable to retrieve DevicePublicKey')
@@ -313,21 +346,16 @@ class LockdownClient(object):
                        'HostCertificate': cert_pem,
                        'HostID': self.host_id,
                        'RootCertificate': cert_pem,
-                       'SystemBUID': '30142955-444094379208051516'}
+                       'SystemBUID': self.system_buid}
 
-        pair = self.service.send_recv_plist({'Label': self.label, 'Request': 'Pair', 'PairRecord': pair_record,
-                                             'ProtocolVersion': '2', 'PairingOptions': {'ExtendedPairingErrors': True}})
+        pair_options = {'PairRecord': pair_record, 'ProtocolVersion': '2',
+                        'PairingOptions': {'ExtendedPairingErrors': True}}
 
-        if pair.get('Error') == 'PasswordProtected':
-            self.service.close()
-            raise NotTrustedError()
-        elif pair.get('Result') != 'Success' and 'EscrowBag' not in pair:
-            self.logger.error(pair.get('Error'))
-            self.service.close()
-            raise PairingError()
+        pair = self._request_pair(pair_options, timeout=timeout)
 
         pair_record['HostPrivateKey'] = private_key_pem
         pair_record['EscrowBag'] = pair.get('EscrowBag')
+        self.pair_record = pair_record
         write_home_file(f'{self.identifier}.plist', plistlib.dumps(pair_record))
 
         record_data = plistlib.dumps(pair_record)
@@ -335,27 +363,24 @@ class LockdownClient(object):
         client = usbmux.create_mux()
         if isinstance(client, PlistMuxConnection):
             client.save_pair_record(self.udid, self.usbmux_device.devid, record_data)
+        client.close()
 
         self.paired = True
 
     @reconnect_on_remote_close
     def unpair(self) -> Mapping:
-        req = {'Label': self.label, 'Request': 'Unpair', 'PairRecord': self.pair_record, 'ProtocolVersion': '2'}
-        return self.service.send_recv_plist(req)
+        return self._request('Unpair', {'PairRecord': self.pair_record, 'ProtocolVersion': '2'}, verify_request=False)
 
     @reconnect_on_remote_close
-    def get_value(self, domain=None, key=None):
-        if isinstance(key, str) and hasattr(self, 'record') and hasattr(self.pair_record, key):
-            return self.pair_record[key]
-
-        req = {'Label': self.label, 'Request': 'GetValue'}
+    def get_value(self, domain: str = None, key: str = None):
+        options = {}
 
         if domain:
-            req['Domain'] = domain
+            options['Domain'] = domain
         if key:
-            req['Key'] = key
+            options['Key'] = key
 
-        res = self.service.send_recv_plist(req)
+        res = self._request('GetValue', options)
         if res:
             r = res.get('Value')
             if hasattr(r, 'data'):
@@ -363,36 +388,42 @@ class LockdownClient(object):
             return r
 
     @reconnect_on_remote_close
-    def set_value(self, value, domain=None, key=None):
-        req = {'Request': 'SetValue', 'Label': self.label}
+    def remove_value(self, domain: str = None, key: str = None) -> Mapping:
+        options = {}
 
         if domain:
-            req['Domain'] = domain
+            options['Domain'] = domain
         if key:
-            req['Key'] = key
+            options['Key'] = key
 
-        req['Value'] = value
-        self.service.send_plist(req)
-        response = self.service.recv_plist()
-        self.logger.debug(response)
-        return response
+        return self._request('RemoveValue', options)
 
-    def get_service_connection_attributes(self, name, escrow_bag=None) -> dict:
+    @reconnect_on_remote_close
+    def set_value(self, value, domain: str = None, key: str = None) -> Mapping:
+        options = {}
+
+        if domain:
+            options['Domain'] = domain
+        if key:
+            options['Key'] = key
+
+        options['Value'] = value
+        return self._request('SetValue', options)
+
+    def get_service_connection_attributes(self, name, escrow_bag=None) -> Mapping:
         if not self.paired:
-            self.logger.info('NotPaired')
             raise NotPairedError()
 
-        request = {'Label': self.label, 'Request': 'StartService', 'Service': name}
+        options = {'Service': name}
         if escrow_bag is not None:
-            request['EscrowBag'] = escrow_bag
+            options['EscrowBag'] = escrow_bag
 
-        self.service.send_plist(request)
-        response = self.service.recv_plist()
+        response = self._request('StartService', options)
         if not response or response.get('Error'):
             if response.get('Error', '') == 'PasswordProtected':
                 raise PasswordRequiredError(
                     'your device is protected with password, please enter password in device and try again')
-            raise StartServiceError(response.get("Error"))
+            raise StartServiceError(response.get('Error'))
         return response
 
     @reconnect_on_remote_close
@@ -424,3 +455,64 @@ class LockdownClient(object):
 
     def close(self):
         self.service.close()
+
+    def _request(self, request: str, options: Mapping = None, verify_request: bool = True) -> Mapping:
+        message = {'Label': self.label, 'Request': request}
+        if options:
+            message.update(options)
+        response = self.service.send_recv_plist(message)
+
+        if verify_request and response['Request'] != request:
+            raise LockdownError(f'incorrect response returned. got {response["Request"]} instead of {request}')
+
+        error = response.get('Error')
+        if error is not None:
+            exception_errors = {'PasswordProtected': PasswordRequiredError,
+                                'PairingDialogResponsePending': PairingDialogResponsePendingError,
+                                'UserDeniedPairing': UserDeniedPairingError,
+                                'InvalidHostID': InvalidHostIDError,
+                                'SetProhibited': SetProhibitedError,
+                                'MissingValue': MissingValueError, }
+            raise exception_errors.get(error, LockdownError)(error)
+
+        # iOS < 5: 'Error' is not present, so we need to check the 'Result' instead
+        if response.get('Result') == 'Failure':
+            raise LockdownError()
+
+        return response
+
+    def _request_pair(self, pair_options: Mapping, timeout: int = None):
+        try:
+            return self._request('Pair', pair_options)
+        except PairingDialogResponsePendingError:
+            if timeout == 0:
+                raise
+
+        self.logger.info('waiting user pairing dialog...')
+        start = time.time()
+        while timeout is None or time.time() <= start + timeout:
+            with suppress(PairingDialogResponsePendingError):
+                return self._request('Pair', pair_options)
+            time.sleep(1)
+        raise PairingDialogResponsePendingError()
+
+    def _init_preferred_pair_record(self):
+        if self.pair_record is not None:
+            # if already have one, use it
+            return
+
+        pair_record = self.get_itunes_pairing_record()
+        if pair_record is not None:
+            self.logger.debug(f'Using iTunes pair record: {self.identifier}.plist')
+
+        mux = usbmux.create_mux()
+        if isinstance(mux, PlistMuxConnection):
+            pair_record = mux.get_pair_record(self.udid)
+        else:
+            pair_record = self.get_local_pairing_record()
+        mux.close()
+
+        if pair_record is None:
+            return
+
+        self.pair_record = pair_record
