@@ -1,20 +1,9 @@
+from typing import List, Mapping, Optional
+
 from pymobiledevice3.exceptions import PyMobileDevice3Exception, ConnectionFailedError
 from pymobiledevice3.lockdown import LockdownClient
 from pymobiledevice3.services.base_service import BaseService
 
-Requests = """Goodbye
-All
-GasGauge
-WiFi
-Shutdown
-Restart
-MobileGestalt
-Sleep
-NAND
-IORegistry
-Obliterate
-"""
-
 MobileGestaltKeys = ['BasebandKeyHashInformation',
                      'BasebandFirmwareManifestData',
                      'DieId',
@@ -116,25 +105,24 @@ class DiagnosticsService(BaseService):
             service_name = self.SERVICE_NAME_OLD
 
         super().__init__(lockdown, service_name, service=service)
-        self.packet_num = 0
 
-    def mobilegestalt(self, keys=None):
+    def mobilegestalt(self, keys: List[str] = None) -> Mapping:
         if keys is None or len(keys) == 0:
             keys = MobileGestaltKeys
-        self.service.send_plist({'Request': 'MobileGestalt',
-                                 'MobileGestaltKeys': keys})
+        resp = self.service.send_recv_plist({'Request': 'MobileGestalt', 'MobileGestaltKeys': keys})
 
-        res = self.service.recv_plist()
-        d = res.get('Diagnostics')
-        if d:
-            return d.get('MobileGestalt')
-        return None
+        if (resp['Status'] != 'Success') or (resp['Diagnostics']['MobileGestalt']['Status'] != 'Success'):
+            raise PyMobileDevice3Exception('failed to query MobileGestalt')
 
-    def action(self, action):
-        self.service.send_plist({'Request': action})
-        response = self.service.recv_plist()
-        if response.get('Status', None) is None:
-            raise PyMobileDevice3Exception(f'got invalid response: {response}')
+        resp['Diagnostics']['MobileGestalt'].pop('Status')
+
+        return resp['Diagnostics']['MobileGestalt']
+
+    def action(self, action: str) -> Optional[Mapping]:
+        response = self.service.send_recv_plist({'Request': action})
+        if response['Status'] != 'Success':
+            raise PyMobileDevice3Exception(f'failed to perform action: {action}')
+        return response.get('Diagnostics')
 
     def restart(self):
         self.action('Restart')
@@ -145,14 +133,10 @@ class DiagnosticsService(BaseService):
     def sleep(self):
         self.action('Sleep')
 
-    def info(self, diag_type='All'):
-        self.service.send_plist({'Request': diag_type})
-        res = self.service.recv_plist()
-        if res:
-            return res.get('Diagnostics')
-        return None
+    def info(self, diag_type: str = 'All') -> Mapping:
+        return self.action(diag_type)
 
-    def ioregistry(self, plane=None, name=None, ioclass=None):
+    def ioregistry(self, plane: str = None, name: str = None, ioclass: str = None):
         d = {}
 
         if plane:
@@ -175,5 +159,5 @@ class DiagnosticsService(BaseService):
             return dd.get('IORegistry')
         return None
 
-    def get_battery(self):
+    def get_battery(self) -> Mapping:
         return self.ioregistry(ioclass='IOPMPowerSource')
